unit XINIFileXE2;

{     Xaid XINIFile 1.0
      Aid Vllasaliu

      Please visit the official Xaid YouTube channel for video instructions.
      They are short, informative and easier to understand than a text manual.

      Donate a couple of Euros if you like what we develop.

      Any type of support is highly appreciated.

      Donate through the link below
      https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=E6E4NBE7CDBQG

      Youtube: http://www.youtube.com/user/xaidcorp
      Sourceforge: https://sourceforge.net/p/xinifile/

      License can be found at https://sourceforge.net/p/xaidlicense/

      Contact: xaidcorp@gmail.com
}

interface

uses
  System.SysUtils, System.Classes, vcl.dialogs;

type
  TIdentArray = array of array [0..1] of string;

  TSection = record
    name   : string;
    idents : TIdentArray;
  end;

  TSectionArray = array of TSection;

  TXINIFile = class(TComponent)
  private
    fINIFile    : TStringList;
    fSections   : TSectionArray;
    fAutoSave   : boolean;
    fFilePath   : String;

    fOnUpdate   : TNotifyEvent;

    function GetSection(index: integer): TSection;
    procedure SetSection(index: integer; value: TSection);

    procedure NotifyUpdate;
  protected
    { Protected declarations }
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;

    procedure LoadINI(INIFilePath: string);
    procedure SaveINI(INIFilePath: String);

    function SectionCount: integer;
    function SectionExists(Section: string): integer;
    function IdentCount(Section: string): integer;
    function IdentExists(Section, Ident: string): integer;

    function SectionsToString: string;
    function IdentsToString(Section: string): string;

    function ReadString(Section, Ident: String): String;
    procedure WriteString(Section, Ident, Value: String);

    procedure RemoveSection(Section: string);
    procedure RemoveIdent(Section, Ident: string);

    function RawData: string;

    property Section[index: integer]: TSection read GetSection write SetSection;
  published
    property AutoSave: boolean read fAutoSave write fAutoSave;
    property FilePath: string read fFilePath write fFilePath;

    property OnUpdate: TNotifyEvent read fOnUpdate write fOnUpdate;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Xaid', [TXINIFile]);
end;

constructor TXINIFile.Create(Owner: TComponent);
begin
  inherited;
  fINIFile:=TStringList.Create;
  fFilePath:='c:\xinifile.ini';
end;

destructor TXINIFile.Destroy;
begin
  fINIFile.Free;
  inherited;
end;

procedure TXINIFile.LoadINI(INIFilePath: String); //WORKING!
  var i,ii, sI: integer;
      str, l: string;

  function IsSection(LineStr: string): boolean;
  begin
    if (length(str)>0) and (str[1]='[') and (str[length(str)]=']') and not (str='') then result:=true;
  end;

begin
  if INIFIlePath='' then exit;

  //Load
  fINIFile.LoadFromFile(INIFilePath);
  if fINIFile.Count=0 then exit;
  fFilepath:=INIFilePath;

  //Parse
  sI:=-1;
  i:=0;
  while i<fINIFile.Count-1 do
  begin
    str:=fINIFile.Strings[i];
    if IsSection(str)=true then
    begin
      sI:=sI+1;
      SetLength(fSections,sI+1);
      fSections[sI].name:=Copy(str,2,length(str)-2);
      for ii := i+1 to fINIFile.Count-1 do
      begin
        str:=fINIFile.Strings[ii];
        if IsSection(str)=true then
         break
        else
          if str<>'' then
          begin
            SetLength(fSections[sI].idents,length(fSections[sI].idents)+1);
            fSections[sI].idents[length(fSections[sI].idents)-1][0]:=Copy(str,1,pos('=',str)-1);
            fSections[sI].idents[length(fSections[sI].idents)-1][1]:=Copy(str,pos('=',str)+1,length(str));
          end;
      end;
    end;

    i:=i+1;
  end;
end;

procedure TXINIFile.SaveINI(INIFilePath: String);
begin
  fFilePath:=INIFilePath;

  if fINIFile.Count=0 then exit;

  fINIFile.SaveToFile(fFilePath);
end;

function TXINIFile.SectionCount: integer;
begin
  result:=length(fSections);
end;

function TXINIFile.IdentCount(Section: string): integer;
 var c: integer;
begin
  c:=SectionExists(Section);
  if c>-1 then result:=length(fSections[c].idents);
end;

function TXINIFile.SectionExists(Section: string): integer;
  var i: integer;
begin
  result:=-1;
  for i := 0 to SectionCount-1 do if fSections[i].name=Section then result:=i;
end;

function TXINIFile.IdentExists(Section, Ident: string): integer;
  var i, ii: integer;
begin
  result:=-1;
  for i := 0 to SectionCount-1 do
    if fSections[i].name=Section then
     for ii := 0 to IdentCount(Section)-1 do
       if fSections[i].idents[ii][0]=Ident then result:=ii;
end;

function TXINIFile.SectionsToString: string;
  var i: integer;
begin
  for i := 0 to SectionCount-1 do
  begin
    result:=result+fSections[i].name;
    if i<SectionCount-1 then result:=result+#13;
  end;
end;

function TXINIFile.IdentsToString(Section: string): string;
  var i, cS,cI: integer;
begin
  cS:=SectionExists(Section);
  if cS>-1 then
    cI:=IdentCount(Section);
    for i := 0 to cI-1 do
    begin
      result:=result+fSections[cS].idents[i][0];
      if i<cI-1 then result:=result+#13;
    end;
end;

function TXINIFile.ReadString(Section, Ident: String): String;
  var cS, cI: integer;
begin
  cS:=SectionExists(Section);
  cI:=IdentExists(Section, Ident);
  if cI>-1 then result:=fSections[cS].idents[cI][1];
end;

procedure TXINIFile.WriteString(Section, Ident, Value: String);
  var cS, cI: integer;
begin
  if (Section='') or (Ident='') then exit;

  cS:=SectionExists(Section);
  if cS=-1 then
  begin
    SetLength(fSections, length(fSections)+1);
    fSections[length(fSections)-1].name:=Section;
    cS:=SectionExists(Section);
  end;

  cI:=IdentExists(Section, Ident);
  if cI=-1 then
  begin
    SetLength(fSections[cS].idents, length(fSections[cS].idents)+1);
    fSections[cS].idents[length(fSections[cS].idents)-1][0]:=Ident;
    cI:=IdentExists(Section, Ident);
  end;

  fSections[cS].idents[cI][1]:=Value;

  NotifyUpdate;
end;

procedure TXINIFile.RemoveSection(Section: string);
  var cS, i: integer;
      ALength: Cardinal;
begin
  cS:=SectionExists(Section);
  if cS=-1 then exit;

  Assert(Length(fSections) > 0);
  Assert(cS < Length(fSections));
  for i := cS + 1 to Length(fSections) - 1 do fSections[i - 1] := fSections[i];
  SetLength(fSections, Length(fSections) - 1);

  NotifyUpdate;
end;

procedure TXINIFile.RemoveIdent(Section, Ident: string);
  var cS, cI, i: integer;
      ALength: Cardinal;
begin
  cS:=SectionExists(Section);
  if cS=-1 then exit;

  cI:=IdentExists(Section, Ident);
  if cI=-1 then exit;

  Assert(Length(fSections[cS].idents) > 0);
  Assert(cS < Length(fSections[cS].idents));
  for i := cI + 1 to Length(fSections[cS].idents) - 1 do fSections[cS].idents[i - 1] := fSections[cS].idents[i];
  SetLength(fSections[cS].idents, Length(fSections[cS].idents) - 1);

  NotifyUpdate;
end;


function TXINIFile.RawData: string;
begin
  result:=fINIFile.Text;
end;

function TXINIFile.GetSection(index: integer): TSection;
begin
  result:=fSections[index];
end;

procedure TXINIFile.SetSection(index: integer; value: TSection);
begin
  fSections[index]:=value;
end;

procedure TXINIFile.NotifyUpdate;
  var i, ii: integer;
begin
  fINIFile.Clear;

  for i := 0 to SectionCount-1 do
  begin
    fINIFile.Add('['+fSections[i].name+']');
    for ii := 0 to length(fSections[i].idents)-1 do
      fINIFile.Add(fSections[i].idents[ii][0]+'='+fSections[i].idents[ii][1]);

    fINIFile.Add('');
  end;

  if Assigned(fOnUpdate) then fOnUpdate(self);

  if fAutoSave=true then SaveINI(fFilePath);
end;

end.
